<?php

require_once "phpfspot_cfg.php";
require_once "phpfspot_db.php";
require_once "phpfspot_tmpl.php";

class PHPFSPOT {

   var $cfg;
   var $db;
   var $cfg_db;
   var $tmpl;
   var $tags;
   var $avail_tags;
   var $current_tags;

   public function __construct()
   {
      /* Check necessary requirements */
      if(!$this->checkRequirements()) {
         exit(1);
      }

      $this->cfg = new PHPFSPOT_CFG;

      $this->db  = new PHPFSPOT_DB(&$this, $this->cfg->fspot_db);
      $this->cfg_db = new PHPFSPOT_DB(&$this, $this->cfg->phpfspot_db);
      $this->check_config_table();

      $this->tmpl = new PHPFSPOT_TMPL($this);

      $this->get_tags();

      session_start();

      if(!isset($_SESSION['tag_condition']))
         $_SESSION['tag_condition'] = 'or';

      if(!isset($_SESSION['searchfor']))
         $_SESSION['searchfor'] = '';

      // if begin_with is still set but rows_per_page is now 0, unset it
      if(isset($_SESSION['begin_with']) && $this->cfg->rows_per_page == 0)
         unset($_SESSION['begin_with']);

   } // __construct()

   public function __destruct()
   {

   } // __destruct()

   public function show()
   {
      $this->tmpl->assign('searchfor', $_SESSION['searchfor']);
      $this->tmpl->assign('page_title', $this->cfg->page_title);
      $this->tmpl->assign('current_condition', $_SESSION['tag_condition']);
      $this->tmpl->assign('from_date', $this->get_calendar());
      $this->tmpl->assign('to_date', $this->get_calendar());

      switch($_GET['mode']) {
         case 'showpi':
            if(isset($_GET['tags'])) {
               $_SESSION['selected_tags'] = split(',', $_GET['tags']);
            }
            break;
         case 'export':
            $this->tmpl->show("export.tpl");
            return;
            break;

      }

      $this->tmpl->assign('content_page', 'welcome.tpl');
      $this->tmpl->show("index.tpl");


   } // show()

   private function get_tags()
   {
   
      $this->avail_tags = Array();
      $count = 0;
   
      $result = $this->db->db_query("
         SELECT id,name
         FROM tags
         ORDER BY sort_priority ASC
      ");
      
      while($row = $this->db->db_fetch_object($result)) {

         $tag_id = $row['id'];
         $tag_name = $row['name'];

         /* check if config requests to ignore this tag */
         if(in_array($row['name'], $this->cfg->hide_tags))
            continue;

         $this->tags[$tag_id] = $tag_name; 
         $this->avail_tags[$count] = $tag_id;

         $count++;

      }

   } // get_tags()

   public function get_photo_details($idx)
   {
      $result = $this->db->db_query("
         SELECT *
         FROM photos
         WHERE id='". $idx ."'
      ");
      
      return $this->db->db_fetch_object($result);

   } // get_photo_details

   public function getPhotoName($idx, $limit = 0)
   {
      if($details = $this->get_photo_details($idx)) {
         $name = $details['name'];
         if($limit != 0 && strlen($name) > $limit) {
            $name = substr($name, 0, $limit-5) ."...". substr($name, -($limit-5));
         }
         return $name;
      }

   } // getPhotoName()

   public function translate_path($path, $width = 0)
   {  
      return str_replace($this->cfg->path_replace_from, $this->cfg->path_replace_to, $path);

   } // translate_path

   public function showPhoto($photo)
   {
      $all_photos = $this->getPhotoSelection();
      $count = count($all_photos);

      for($i = 0; $i < $count; $i++) {
         
         if($get_next) {
            $next_img = $all_photos[$i];
            break;
         }

         if($all_photos[$i] == $photo) {
            $get_next = 1;
         }
         else {
            $previous_img = $all_photos[$i];
         }

         if($photo == $all_photos[$i]) {
               $current = $i;
         }
      }

      $details = $this->get_photo_details($photo);
      $orig_path = $this->translate_path($details['directory_path']) ."/". $details['name'];
      $thumb_path = $this->cfg->base_path ."/thumbs/". $this->cfg->photo_width ."_". $this->getMD5($photo);

      if(!file_exists($orig_path)) {
         $this->_warning("Photo ". $orig_path ." does not exist!<br />\n");
      }

      if(!is_readable($orig_path)) {
         $this->_warning("Photo ". $orig_path ." is not readable for user ". get_current_user() ."<br />\n");
      }

      /* If the thumbnail doesn't exist yet, try to create it */
      if(!file_exists($thumb_path)) {
         $this->gen_thumb($photo, true);
      }

      $meta = $this->get_meta_informations($orig_path);

      /* If EXIF data are available, use them */
      if(isset($meta['ExifImageWidth'])) {
         $meta_res = $meta['ExifImageWidth'] ."x". $meta['ExifImageLength'];
      } else {
         $info = getimagesize($orig_path);
         $meta_res = $info[0] ."x". $info[1]; 
      }

      $meta_date = isset($meta['FileDateTime']) ? strftime("%a %x %X", $meta['FileDateTime']) : "n/a";
      $meta_make = isset($meta['Make']) ? $meta['Make'] ." ". $meta['Model'] : "n/a";
      $meta_size = isset($meta['FileSize']) ? round($meta['FileSize']/1024, 1) ."kbyte" : "n/a";

      if(file_exists($thumb_path)) {

         $info = getimagesize($thumb_path);

         $this->tmpl->assign('description', $details['description']);
         $this->tmpl->assign('image_name', $details['name']);

         $this->tmpl->assign('width', $info[0]);
         $this->tmpl->assign('height', $info[1]);
         $this->tmpl->assign('ExifMadeOn', $meta_date);
         $this->tmpl->assign('ExifMadeWith', $meta_make);
         $this->tmpl->assign('ExifOrigResolution', $meta_res);
         $this->tmpl->assign('ExifFileSize', $meta_size);
    
         $this->tmpl->assign('image_url', 'phpfspot_img.php?idx='. $photo ."&amp;width=". $this->cfg->photo_width);
         $this->tmpl->assign('image_url_full', 'phpfspot_img.php?idx='. $photo);

         $this->tmpl->assign('tags', $this->get_photo_tags($photo));
         $this->tmpl->assign('current', $current);
      }
      else {
         $this->_warning("Can't open file ". $thumb_path ."\n");
      }

      if($previous_img) {
         $this->tmpl->assign('previous_url', "javascript:showImage(". $previous_img .");");
         $this->tmpl->assign('prev_img', $previous_img);
      }

      if($next_img) {
         $this->tmpl->assign('next_url', "javascript:showImage(". $next_img .");");
         $this->tmpl->assign('next_img', $next_img);
      }
      $this->tmpl->assign('mini_width', $this->cfg->mini_width);

      $this->tmpl->show("single_photo.tpl");

   } // showPhoto()

   public function getAvailableTags()
   {
      $result = $this->db->db_query("
         SELECT tag_id as id, count(tag_id) as quantity
         FROM photo_tags
         INNER JOIN tags t
            ON t.id = tag_id
         GROUP BY tag_id
         ORDER BY t.name ASC
      ");

      $tags = Array();

      while($row = $this->db->db_fetch_object($result)) {
         $tags[$row['id']] = $row['quantity'];
      }

      // change these font sizes if you will
      $max_size = 125; // max font size in %
      $min_size = 75; // min font size in %

      // get the largest and smallest array values
      $max_qty = max(array_values($tags));
      $min_qty = min(array_values($tags));

      // find the range of values
      $spread = $max_qty - $min_qty;
      if (0 == $spread) { // we don't want to divide by zero
         $spread = 1;
      }

      // determine the font-size increment
      // this is the increase per tag quantity (times used)
      $step = ($max_size - $min_size)/($spread);

      // loop through our tag array
      foreach ($tags as $key => $value) {

         if(isset($_SESSION['selected_tags']) && in_array($key, $_SESSION['selected_tags']))
            continue;

          // calculate CSS font-size
          // find the $value in excess of $min_qty
          // multiply by the font-size increment ($size)
          // and add the $min_size set above
         $size = $min_size + (($value - $min_qty) * $step);
          // uncomment if you want sizes in whole %:
         $size = ceil($size);

         print "<a href=\"javascript:Tags('add', ". $key .");\" class=\"tag\" style=\"font-size: ". $size ."%;\">". $this->tags[$key] ."</a>, ";

      }

   } // getAvailableTags()

   public function getSelectedTags()
   {
      $output = "";
      foreach($this->avail_tags as $tag)
      {
         // return all selected tags
         if(isset($_SESSION['selected_tags']) && in_array($tag, $_SESSION['selected_tags'])) {
            $output.= "<a href=\"javascript:Tags('del', ". $tag .");\" class=\"tag\">". $this->tags[$tag] ."</a>, ";
         }
      }

      $output = substr($output, 0, strlen($output)-2);
      print $output;

   } // getSelectedTags()

   public function addTag($tag)
   {
      if(!isset($_SESSION['selected_tags']))
         $_SESSION['selected_tags'] = Array();

      if(!in_array($tag, $_SESSION['selected_tags']))
         array_push($_SESSION['selected_tags'], $tag);
   
   } // addTag()

   public function delTag($tag)
   {
      if(isset($_SESSION['selected_tags'])) {
         $key = array_search($tag, $_SESSION['selected_tags']);
         unset($_SESSION['selected_tags'][$key]);
         sort($_SESSION['selected_tags']);
      }

   } // delTag()

   public function resetTags()
   {
      unset($_SESSION['selected_tags']);

   } // resetTags()

   public function resetTagSearch()
   {
      unset($_SESSION['searchfor']);

   } // resetTagSearch()

   public function getPhotoSelection()
   {  
      $tagged_photos = Array();

      /* return a search result */
      if(isset($_SESSION['searchfor']) && $_SESSION['searchfor'] != '') {
         $result = $this->db->db_query("
            SELECT DISTINCT photo_id
               FROM photo_tags pt
            INNER JOIN photos p
               ON p.id=pt.photo_id
            INNER JOIN tags t
               ON pt.tag_id=t.id
            WHERE t.name LIKE '%". $_SESSION['searchfor'] ."%'
               ORDER BY p.time ASC
         ");
         while($row = $this->db->db_fetch_object($result)) {
            array_push($tagged_photos, $row['photo_id']);
         }
         return $tagged_photos;
      }

      /* return according the selected tags */
      if(isset($_SESSION['selected_tags']) && !empty($_SESSION['selected_tags'])) {
         $selected = "";
         foreach($_SESSION['selected_tags'] as $tag)
            $selected.= $tag .",";
         $selected = substr($selected, 0, strlen($selected)-1);

         if($_SESSION['tag_condition'] == 'or') {
            $result = $this->db->db_query("
               SELECT DISTINCT photo_id
                  FROM photo_tags pt
               INNER JOIN photos p
                  ON p.id=pt.photo_id
               WHERE pt.tag_id IN (". $selected .")
               ORDER BY p.time ASC
            ");
         }
         elseif($_SESSION['tag_condition'] == 'and') {

            if(count($_SESSION['selected_tags']) >= 32) {
               print "A SQLite limit of 32 tables within a JOIN SELECT avoids to<br />\n";
               print "evaluate your tag selection. Please remove some tags from your selection.\n";
               return Array();
            } 

            /* Join together a table looking like

               pt1.photo_id pt1.tag_id pt2.photo_id pt2.tag_id ...

               so the query can quickly return all images matching the
               selected tags in an AND condition

            */

            $query_str = "
               SELECT DISTINCT pt1.photo_id
                  FROM photo_tags pt1
            ";

            for($i = 0; $i < count($_SESSION['selected_tags']); $i++) {
               $query_str.= "
                  INNER JOIN photo_tags pt". ($i+2) ."
                     ON pt1.photo_id=pt". ($i+2) .".photo_id
               ";
            }
            $query_str.= "WHERE pt1.tag_id=". $_SESSION['selected_tags'][0];
            for($i = 1; $i < count($_SESSION['selected_tags']); $i++) {
               $query_str.= "
                  AND pt". ($i+1) .".tag_id=". $_SESSION['selected_tags'][$i] ."
               "; 
            }
            $result = $this->db->db_query($query_str);
         }

         while($row = $this->db->db_fetch_object($result)) {
            array_push($tagged_photos, $row['photo_id']);
         }
         return $tagged_photos;
      }

      /* return all available photos */
      $result = $this->db->db_query("
         SELECT DISTINCT photo_id
            FROM photo_tags pt
         INNER JOIN photos p
            ON p.id=pt.photo_id
         ORDER BY p.time ASC
      ");
      while($row = $this->db->db_fetch_object($result)) {
         array_push($tagged_photos, $row['photo_id']);
      }
      return $tagged_photos;

   } // getPhotoSelection()

   public function showPhotoIndex()
   {
      $photos = $this->getPhotoSelection();

      $count = count($photos);

      if(isset($_SESSION['begin_with']) && $_SESSION['begin_with'] != "")
         $anchor = $_SESSION['begin_with'];

      if(!isset($this->cfg->rows_per_page) || $this->cfg->rows_per_page == 0) {

         $begin_with = 0;
         $end_with = $count;

      }
      elseif($this->cfg->rows_per_page > 0) {

         if(!$_SESSION['begin_with'] || $_SESSION['begin_with'] == 0)
            $begin_with = 0;
         else {

            $begin_with = $_SESSION['begin_with'];

            // verify $begin_with - perhaps the thumbs-per-rows or
            // rows-per-page variables have changed or the jump back
            // from a photo wasn't exact - so calculate the real new
            // starting point
            $multiplicator = $this->cfg->rows_per_page * $this->cfg->thumbs_per_row;
            for($i = 0; $i <= $count; $i+=$multiplicator) {
               if($begin_with >= $i && $begin_with < $i+$multiplicator) {
                  $begin_with = $i;
                  break;
               }
            }
         }

         $end_with = $begin_with + ($this->cfg->rows_per_page * $this->cfg->thumbs_per_row);
      }

   
      $rows = 0;
      $cols = 0;
      $images[$rows] = Array();
      $img_height[$rows] = Array();
      $img_width[$rows] = Array();
      $img_id[$rows] = Array();
      $img_name[$rows] = Array();
      $img_title = Array();

      for($i = $begin_with; $i < $end_with; $i++) {

         $images[$rows][$cols] = $photos[$i];
         $img_id[$rows][$cols] = $i;
         $img_name[$rows][$cols] = htmlspecialchars($this->getPhotoName($photos[$i], 15));
         $img_title[$rows][$cols] = "Click to view photo ". htmlspecialchars($this->getPhotoName($photos[$i], 0));

         $thumb_path = $this->cfg->base_path ."/thumbs/". $this->cfg->thumb_width ."_". $this->getMD5($photos[$i]);

         if(file_exists($thumb_path)) {
            $info = getimagesize($thumb_path); 
            $img_width[$rows][$cols] = $info[0];
            $img_height[$rows][$cols] = $info[1];
         }

         if($cols == $this->cfg->thumbs_per_row-1) {
            $cols = 0;
            $rows++;
            $images[$rows] = Array();
            $img_width[$rows] = Array();
            $img_height[$rows] = Array();
         }
         else {
            $cols++;
         }
      } 

      // +1 for for smarty's selection iteration
      $rows++;

      if(isset($_SESSION['searchfor']) && $_SESSION['searchfor'] != '')
         $this->tmpl->assign('searchfor', $_SESSION['searchfor']);

      /* do we have to display the page selector ? */
      if($this->cfg->rows_per_page != 0) {
      
         /* calculate the page switchers */
         $previous_start = $begin_with - ($this->cfg->rows_per_page * $this->cfg->thumbs_per_row);
         $next_start = $begin_with + ($this->cfg->rows_per_page * $this->cfg->thumbs_per_row);

         if($begin_with != 0) 
            $this->tmpl->assign("previous_url", "javascript:showPhotoIndex(". $previous_start .");"); 
         if($end_with < $count)
            $this->tmpl->assign("next_url", "javascript:showPhotoIndex(". $next_start .");"); 

         $photo_per_page  = $this->cfg->rows_per_page * $this->cfg->thumbs_per_row;
         $last_page = ceil($count / $photo_per_page);

         /* get the current selected page */
         if($begin_with == 0) {
            $current_page = 1;
         } else {
            $current_page = 0;
            for($i = $begin_with; $i >= 0; $i-=$photo_per_page) {
               $current_page++;
            }
         } 

         for($i = 1; $i <= $last_page; $i++) {

            if($current_page == $i)
               $style = "style=\"font-size: 125%;\"";
            elseif($current_page-1 == $i || $current_page+1 == $i)
               $style = "style=\"font-size: 105%;\"";
            elseif(($current_page-5 >= $i) && ($i != 1) ||
               ($current_page+5 <= $i) && ($i != $last_page))
               $style = "style=\"font-size: 75%;\"";
            else
               $style = "";

            $select = "<a href=\"javascript:showPhotoIndex(". (($i*$photo_per_page)-$photo_per_page) .");\"";
               if($style != "")
                  $select.= $style;
            $select.= ">". $i ."</a>&nbsp;";

            // until 9 pages we show the selector from 1-9
            if($last_page <= 9) {
               $page_select.= $select;
               continue;
            } else {
               if($i == 1 /* first page */ || 
                  $i == $last_page /* last page */ ||
                  $i == $current_page /* current page */ ||
                  $i == ceil($last_page * 0.25) /* first quater */ ||
                  $i == ceil($last_page * 0.5) /* half */ ||
                  $i == ceil($last_page * 0.75) /* third quater */ ||
                  (in_array($i, array(1,2,3,4,5,6)) && $current_page <= 4) /* the first 6 */ ||
                  (in_array($i, array($last_page, $last_page-1, $last_page-2, $last_page-3, $last_page-4, $last_page-5)) && $current_page >= $last_page-4) /* the last 6 */ ||
                  $i == $current_page-3 || $i == $current_page-2 || $i == $current_page-1 /* three before */ ||
                  $i == $current_page+3 || $i == $current_page+2 || $i == $current_page+1 /* three after */) {

                  $page_select.= $select;
                  continue;

               }
            }

            $page_select.= ".";
         }

         /* only show the page selector if we have more then one page */
         if($last_page > 1)
            $this->tmpl->assign('page_selector', $page_select);
      }

      $current_tags = "";
      if($_SESSION['selected_tags'] != "") {
         foreach($_SESSION['selected_tags'] as $tag)
            $current_tags.= $tag .",";
         $current_tags = substr($current_tags, 0, strlen($current_tags)-1);
      }

      $extern_link = "index.php?mode=showpi";
      if($current_tags != "") {
         $extern_link.= "&tags=". $current_tags;
      }

      $export_link = "index.php?mode=export";

      $this->tmpl->assign('extern_link', $extern_link);
      $this->tmpl->assign('export_link', $export_link);
      $this->tmpl->assign('count', $count);
      $this->tmpl->assign('width', $this->cfg->thumb_width);
      $this->tmpl->assign('images', $images);
      $this->tmpl->assign('img_width', $img_width);
      $this->tmpl->assign('img_height', $img_height);
      $this->tmpl->assign('img_id', $img_id);
      $this->tmpl->assign('img_name', $img_name);
      $this->tmpl->assign('img_title', $img_title);
      $this->tmpl->assign('rows', $rows);
      $this->tmpl->assign('columns', $this->cfg->thumbs_per_row);

      $this->tmpl->show("photo_index.tpl");

      if(isset($anchor))
         print "<script language=\"JavaScript\">self.location.hash = '#image". $anchor ."';</script>\n";

   } // showPhotoIndex()

   public function showBubbleDetails($photo, $direction)
   {
      if($direction == "up")
         $direction = "bubbleimg_up";
      else
         $direction = "bubbleimg_down";

      $details = $this->get_photo_details($photo);
      $orig_path = $this->translate_path($details['directory_path'])  ."/". $details['name'];

      $image_url = "phpfspot_img.php?idx=". $photo ."&amp;width=". $this->cfg->bubble_width;

      $filesize = filesize($orig_path);
      $filesize = rand($filesize/1024, 2);

      if(!file_exists($orig_path)) {
         $this->_warning("Photo ". $orig_path ." does not exist!<br />\n");
         return;
      }
      
      if(!is_readable($orig_path)) {
         $this->_warning("Photo ". $orig_path ." is not readable for user ". get_current_user() ."<br />\n");
         return;
      }

      $img = getimagesize($orig_path);

      $this->tmpl->assign('file_size', $filesize);
      $this->tmpl->assign('width', $img[0]);
      $this->tmpl->assign('height', $img[1]);
      $this->tmpl->assign('file_name', $details['name']);
      $this->tmpl->assign('image_id', $direction);
      $this->tmpl->assign('image_url', $image_url);
      $this->tmpl->show("bubble_details.tpl");

   } // showBubbleDetails()

   public function showCredits()
   {
      $this->tmpl->assign('version', $this->cfg->version);
      $this->tmpl->assign('product', $this->cfg->product);
      $this->tmpl->show("credits.tpl");

   } // showCredits()

   public function create_thumbnail($orig_image, $thumb_image, $width)
   {  
      if(!file_exists($orig_image))
         return false;

      $details = getimagesize($orig_image);
      
      /* check if original photo is a support image type */
      if(!$this->checkifImageSupported($details['mime']))
         return false;

      $meta = $this->get_meta_informations($orig_image);

      $rotate = 0;
      $flip = false;

      switch($meta['Orientation']) {

         case 1: /* top, left */
            $rotate = 0; $flip = false; break;
         case 2: /* top, right */
            $rotate = 0; $flip = true; break;
         case 3: /* bottom, left */
            $rotate = 180; $flip = false; break;
         case 4: /* bottom, right */
            $rotate = 180; $flip = true; break;
         case 5: /* left side, top */
            $rotate = 90; $flip = true; break;
         case 6: /* right side, top */
            $rotate = 90; $flip = false; break;
         case 7: /* left side, bottom */
            $rotate = 270; $flip = true; break;
         case 8: /* right side, bottom */
            $rotate = 270; $flip = false; break;
      }

      $src_img = @imagecreatefromjpeg($orig_image);

      if(!$src_img) {
         print "Can't load image from ". $orig_image ."\n";
         return false;
      }

      /* grabs the height and width */
      $cur_width = imagesx($src_img);
      $cur_height = imagesy($src_img);

      // If requested width is more then the actual image width,
      // do not generate a thumbnail

      if($width >= $cur_width) {
         imagedestroy($src_img);
         return true;
      }

      // If the image will be rotate because EXIF orientation said so
      // 'virtually rotate' the image for further calculations
      if($rotate == 90 || $rotate == 270) {
         $tmp = $cur_width;
         $cur_width = $cur_height;
         $cur_height = $tmp;
      }

      /* calculates aspect ratio */
      $aspect_ratio = $cur_height / $cur_width;

      /* sets new size */
      if($aspect_ratio < 1) {
         $new_w = $width;
         $new_h = abs($new_w * $aspect_ratio);
      } else {
         /* 'virtually' rotate the image and calculate it's ratio */
         $tmp_w = $cur_height;
         $tmp_h = $cur_width;
         /* now get the ratio from the 'rotated' image */
         $tmp_ratio = $tmp_h/$tmp_w;
         /* now calculate the new dimensions */
         $tmp_w = $width;
         $tmp_h = abs($tmp_w * $tmp_ratio);

         // now that we know, how high they photo should be, if it
         // gets rotated, use this high to scale the image
         $new_h = $tmp_h;
         $new_w = abs($new_h / $aspect_ratio);

         // If the image will be rotate because EXIF orientation said so
         // now 'virtually rotate' back the image for the image manipulation
         if($rotate == 90 || $rotate == 270) {
            $tmp = $new_w;
            $new_w = $new_h;
            $new_h = $tmp;
         }
      }

      /* creates new image of that size */
      $dst_img = imagecreatetruecolor($new_w, $new_h);

      imagefill($dst_img, 0, 0, ImageColorAllocate($dst_img, 255, 255, 255));

      /* copies resized portion of original image into new image */
      imagecopyresampled($dst_img, $src_img, 0, 0, 0, 0, $new_w, $new_h, imagesx($src_img), imagesy($src_img));

      /* needs the image to be flipped horizontal? */
      if($flip) {
         print "(FLIP)";
         $image = $dst_img;
         for($x = 0; $x < $new_w; $x++) {
            imagecopy($dst_img, $image, $x, 0, $w - $x - 1, 0, 1, $h);
         }
      }

      if($rotate) {
         $this->_debug("(ROTATE)");
         $dst_img = $this->rotateImage($dst_img, $rotate);
      }

      /* write down new generated file */
      $result = imagejpeg($dst_img, $thumb_image, 75);

      /* free your mind */
      imagedestroy($dst_img);
      imagedestroy($src_img);

      if($result === false) {
         print "Can't write thumbnail ". $thumb_image ."\n";
         return false;
      }

      return true;

   } // create_thumbnail()

   public function get_meta_informations($file)
   {
      return exif_read_data($file);

   } // get_meta_informations()

   public function check_config_table()
   {
      // if the config table doesn't exist yet, create it
      if(!$this->cfg_db->db_check_table_exists("images")) {
         $this->cfg_db->db_exec("
            CREATE TABLE images (
               img_idx int primary key,
               img_md5 varchar(32)
            )
            ");
      }

   } // check_config_table

   /**
    * Generates a thumbnail from photo idx
    *
    * This function will generate JPEG thumbnails from provided F-Spot photo
    * indizes.
    *
    * 1. Check if all thumbnail generations (width) are already in place and
    *    readable
    * 2. Check if the md5sum of the original file has changed
    * 3. Generate the thumbnails if needed
    */
   public function gen_thumb($idx = 0, $force = 0)
   {
      $error = 0;

      $resolutions = Array(
         $this->cfg->thumb_width,
         $this->cfg->bubble_width,
         $this->cfg->photo_width,
         $this->cfg->mini_width,
      );

      /* get details from F-Spot's database */
      $details = $this->get_photo_details($idx);

      /* calculate file MD5 sum */
      $full_path = $this->translate_path($details['directory_path'])  ."/". $details['name'];

      if(!file_exists($full_path)) {
         $this->_warning("File ". $full_path ." does not exist\n");
         return;
      }

      if(!is_readable($full_path)) {
         $this->_warning("File ". $full_path ." is not readable for ". get_current_user() ."\n");
         return;
      }

      $file_md5 = md5_file($full_path);

      $this->_debug("Image [". $idx ."] ". $details['name'] ." Thumbnails:");

      foreach($resolutions as $resolution) {

         $thumb_path = $this->cfg->base_path ."/thumbs/". $resolution ."_". $file_md5;

         /* if the thumbnail file doesn't exist, create it */
         if(!file_exists($thumb_path)) {

            $this->_debug(" ". $resolution ."px");
            if(!$this->create_thumbnail($full_path, $thumb_path, $resolution))
               $error = 1;
         }

         /* if the file hasn't changed there is no need to regen the thumb */
         elseif($file_md5 != $this->getMD5($idx) || $force) {

            $this->_debug(" ". $resolution ."px");
            if(!$this->create_thumbnail($full_path, $thumb_path, $resolution))
               $error = 1;

         }
      }

      /* set the new/changed MD5 sum for the current photo */
      if(!$error) {
         $this->setMD5($idx, $file_md5);
      }

      $this->_debug("\n");

   } // gen_thumb()

   public function getMD5($idx)
   {
      $result = $this->cfg_db->db_query("
         SELECT img_md5 
         FROM images
         WHERE img_idx='". $idx ."'
      ");

      if(!$result)
         return 0;

      $img = $this->cfg_db->db_fetch_object($result);
      return $img['img_md5'];
      
   } // getMD5()

   private function setMD5($idx, $md5)
   {
      $result = $this->cfg_db->db_exec("
         REPLACE INTO images (img_idx, img_md5)
         VALUES ('". $idx ."', '". $md5 ."')
      ");

   } // setMD5()

   public function setTagCondition($mode)
   {
      $_SESSION['tag_condition'] = $mode;

   } // setTagCondition()

   public function startTagSearch($searchfor)
   {
      $_SESSION['searchfor'] = $searchfor;
      $_SESSION['selected_tags'] = Array();

      foreach($this->avail_tags as $tag) {
         if(preg_match('/'. $searchfor .'/i', $this->tags[$tag]))
            array_push($_SESSION['selected_tags'], $tag);
      }

   } // startTagSearch()

   private function rotateImage($img, $degrees)
   {
      if(function_exists("imagerotate"))
         $img = imagerotate($img, $degrees, 0);
      else
      {
         function imagerotate($src_img, $angle)
         {
            $src_x = imagesx($src_img);
            $src_y = imagesy($src_img);
            if ($angle == 180) {
               $dest_x = $src_x;
               $dest_y = $src_y;
            }
            elseif ($src_x <= $src_y) {
               $dest_x = $src_y;
               $dest_y = $src_x;
            }
            elseif ($src_x >= $src_y) {
               $dest_x = $src_y;
               $dest_y = $src_x;
            }
               
            $rotate=imagecreatetruecolor($dest_x,$dest_y);
            imagealphablending($rotate, false);
               
            switch ($angle) {
            
               case 90:
                  for ($y = 0; $y < ($src_y); $y++) {
                     for ($x = 0; $x < ($src_x); $x++) {
                        $color = imagecolorat($src_img, $x, $y);
                        imagesetpixel($rotate, $dest_x - $y - 1, $x, $color);
                     }
                  }
                  break;

               case 270:
                  for ($y = 0; $y < ($src_y); $y++) {
                     for ($x = 0; $x < ($src_x); $x++) {
                        $color = imagecolorat($src_img, $x, $y);
                        imagesetpixel($rotate, $y, $dest_y - $x - 1, $color);
                     }
                  }
                  break;

               case 180:
                  for ($y = 0; $y < ($src_y); $y++) {
                     for ($x = 0; $x < ($src_x); $x++) {
                        $color = imagecolorat($src_img, $x, $y);
                        imagesetpixel($rotate, $dest_x - $x - 1, $dest_y - $y - 1, $color);
                     }
                  }
                  break;

               default: $rotate = $src_img;
            };

            return $rotate;

         }

         $img = imagerotate($img, $degrees);

      }

      return $img;

   } // rotateImage()

   private function get_photo_tags($idx)
   {
      $result = $this->db->db_query("
         SELECT t.id, t.name
         FROM tags t
         INNER JOIN photo_tags pt
            ON t.id=pt.tag_id
         WHERE pt.photo_id='". $idx ."'
      ");

      $tags = Array();

      while($row = $this->db->db_fetch_object($result))
         $tags[$row['id']] = $row['name'];

      return $tags;

   } // get_photo_tags()

   public function showTextImage($txt, $color=000000, $space=4, $font=4, $w=300)
   {
      if (strlen($color) != 6) 
         $color = 000000;

      $int = hexdec($color);
      $h = imagefontheight($font);
      $fw = imagefontwidth($font);
      $txt = explode("\n", wordwrap($txt, ($w / $fw), "\n"));
      $lines = count($txt);
      $im = imagecreate($w, (($h * $lines) + ($lines * $space)));
      $bg = imagecolorallocate($im, 255, 255, 255);
      $color = imagecolorallocate($im, 0xFF & ($int >> 0x10), 0xFF & ($int >> 0x8), 0xFF & $int);
      $y = 0;

      foreach ($txt as $text) {
         $x = (($w - ($fw * strlen($text))) / 2);
         imagestring($im, $font, $x, $y, $text, $color);
         $y += ($h + $space);
      }

      Header("Content-type: image/png");
      ImagePng($im);

   } // showTextImage()

   private function checkRequirements()
   {
      if(!function_exists("imagecreatefromjpeg")) {
         print "PHP GD library extension is missing<br />\n";
         $missing = true;
      }

      if(!function_exists("sqlite3_open")) {
         print "PHP SQLite3 library extension is missing<br />\n";
         $missing = true;
      }

      /* Check for HTML_AJAX PEAR package, lent from Horde project */
      ini_set('track_errors', 1);
      @include_once 'HTML/AJAX/Server.php';
      if(isset($php_errormsg) && preg_match('/Failed opening.*for inclusion/i', $php_errormsg)) {
         print "PEAR HTML_AJAX package is missing<br />\n";
         $missing = true;
      }
      @include_once 'Calendar/Calendar.php';
      if(isset($php_errormsg) && preg_match('/Failed opening.*for inclusion/i', $php_errormsg)) {
         print "PEAR Calendar package is missing<br />\n";
         $missing = true;
      }
      ini_restore('track_errors');

      if(isset($missing))
         return false;

      return true;

   } // checkRequirements()

   private function _debug($text)
   {
      if($this->fromcmd) {
         print $text;
      }

   } // _debug()

   public function checkifImageSupported($mime)
   {
      if(in_array($mime, Array("image/jpeg")))
         return true;

      return false;

   } // checkifImageSupported()

   public function _warning($text)
   {
      print "<img src=\"resources/green_info.png\" alt=\"warning\" />\n";
      print $text;

   } // _warning()

   private function get_calendar()
   {
//      require_once CALENDAR_ROOT.'Year.php';
//      require_once CALENDAR_ROOT.'Month.php';
//      require_once CALENDAR_ROOT.'Day.php';

//      $Year = new Calendar_Year();
//      $Month = new Calendar_Month();
//      $Day = new Calendar_Day();

      $output = "<select name=\"year\">\n";
      for($year = 1990; $year <= date("Y"); $year++) {
         $output.= "<option value=\"". $year ."\">". $year ."</option>\n";
      }
      $output.= "</select>\n";

      $output.= "<select name=\"month\">\n";
      for($month = 1; $month <= 12; $month++) {
         $output.= "<option value=\"". $month ."\">". $month ."</option>\n";
      }
      $output.= "</select>\n";

      $output.= "<select name=\"day\">\n";
      for($day = 1; $day <= 31; $day++) {
         $output.= "<option value=\"". $day ."\">". $day ."</option>\n";
      }
      $output.= "</select>\n";

      return $output;

   } // get_calendar()

   public function getExport($mode)
   {
      $pictures = $this->getPhotoSelection();

      if(!isset($_SERVER['HTTPS'])) $protocol = "http";
      else $protocol = "https";

      $server_name = $_SERVER['SERVER_NAME'];

      foreach($pictures as $picture) {

         $orig_url = $protocol ."://". $server_name . $this->cfg->web_path ."phpfspot_img.php?idx=". $picture ."&width=". $this->cfg->photo_width;
         $thumb_url = $protocol ."://". $server_name . $this->cfg->web_path ."phpfspot_img.php?idx=". $picture ."&width=". $this->cfg->thumb_width;

         switch($mode) {

            case 'HTML':
               // <a href="%pictureurl%"><img src="%thumbnailurl%" ></a>
               print htmlspecialchars("<a href=\"". $orig_url ."\"><img src=\"". $thumb_url ."\" /></a>") ."<br />\n";
               break;
               
            case 'MoinMoin':
               // [%pictureurl% %thumbnailurl%]
               print htmlspecialchars(" * [".$orig_url." ".$thumb_url."&fake=1.jpg]") ."<br />\n";
               break;
         }

      }

   } // getExport()

}

?>
